from molnet.utils import Config

molnet_config = {
    'QM7': Config('http://deepchem.io.s3-website-us-west-1.amazonaws.com/datasets/qm7.csv',
                  '85266bf2676a86403863a32c8961fd8c',
                  'smiles',
                  ['u0_atom'], 'regression', '.', ['qm7.csv'], 'stratified', 'MAE'),
    # 'QM7b': Config('https://s3-us-west-1.amazonaws.com/deepchem.io/datasets/molnet_publish/qm7b.zip',
    #                '4094d209daae761f579831f36d52e3ec',
    #                'smiles',
    #                [], 'regression', 'qm7b', ['qm7b.mat'], 'random', 'MAE'),
    'QM8': Config('https://s3-us-west-1.amazonaws.com/deepchem.io/datasets/molnet_publish/qm8.zip',
                  '3445cc0fb829dc36e34e7935e2643cf2',
                  'gdb9_index',  # This is index for SDF file, not smiles
                  ['E1-CC2', 'E2-CC2', 'f1-CC2', 'f2-CC2', 'E1-PBE0', 'E2-PBE0', 'f1-PBE0',
                   'f2-PBE0', 'E1-PBE0', 'E2-PBE0', 'f1-PBE0', 'f2-PBE0', 'E1-CAM', 'E2-CAM',
                   'f1-CAM', 'f2-CAM'],
                  'regression', 'qm8', ['qm8.sdf.csv', 'qm8.sdf'], 'random', 'MAE'),
    'QM9': Config('https://s3-us-west-1.amazonaws.com/deepchem.io/datasets/molnet_publish/qm9.zip',
                  'ac30e79b1e9dec6f55b8bb3cb2bd3363',
                  'mol_id',
                  ['mu', 'alpha', 'homo', 'lumo', 'gap', 'r2',
                   'zpve', 'cv', 'u0', 'u298', 'h298', 'g298'],
                  'regression', 'qm9', ['gdb9.sdf.csv', 'gdb9.sdf'], 'random', 'MAE'),

    'ESOL': Config('https://s3-us-west-1.amazonaws.com/deepchem.io/datasets/molnet_publish/ESOL.zip',
                   'de31fad4345e40a06ed0f59925609dda',
                   'smiles',
                   ['measured log solubility in mols per litre'],
                   'regression', 'ESOL', ['delaney-processed.csv'], 'random', 'RMSE'),
    'FreeSolv': Config('https://s3-us-west-1.amazonaws.com/deepchem.io/datasets/molnet_publish/FreeSolv.zip',
                       '8d681babd239b15e2f8b2d29f025577a',
                       'smiles',
                       ['expt'],
                       'regression', 'FreeSolv', ['SAMPL.csv'], 'random', 'RMSE'),
    'Lipophilicity': Config('https://s3-us-west-1.amazonaws.com/deepchem.io/datasets/molnet_publish/lipophilicity.zip',
                            'a254cada3f128a6ce13a18990775219e',
                            'smiles',
                            ['exp'],
                            'regression', 'lipophilicity', ['Lipophilicity.csv'], 'random', 'RMSE'),

    'PCBA': Config('https://s3-us-west-1.amazonaws.com/deepchem.io/datasets/molnet_publish/pcba.zip',
                   '7fbe4b271bfe7786c7cc696cd4a384a3',
                   'smiles',
                   ['PCBA-1030', 'PCBA-1379', 'PCBA-1452', 'PCBA-1454', 'PCBA-1457',
                    'PCBA-1458', 'PCBA-1460', 'PCBA-1461', 'PCBA-1468', 'PCBA-1469',
                    'PCBA-1471', 'PCBA-1479', 'PCBA-1631', 'PCBA-1634', 'PCBA-1688',
                    'PCBA-1721', 'PCBA-2100', 'PCBA-2101', 'PCBA-2147', 'PCBA-2242',
                    'PCBA-2326', 'PCBA-2451', 'PCBA-2517', 'PCBA-2528', 'PCBA-2546',
                    'PCBA-2549', 'PCBA-2551', 'PCBA-2662', 'PCBA-2675', 'PCBA-2676',
                    'PCBA-411', 'PCBA-463254', 'PCBA-485281', 'PCBA-485290', 'PCBA-485294',
                    'PCBA-485297', 'PCBA-485313', 'PCBA-485314', 'PCBA-485341', 'PCBA-485349',
                    'PCBA-485353', 'PCBA-485360', 'PCBA-485364', 'PCBA-485367', 'PCBA-492947',
                    'PCBA-493208', 'PCBA-504327', 'PCBA-504332', 'PCBA-504333', 'PCBA-504339',
                    'PCBA-504444', 'PCBA-504466', 'PCBA-504467', 'PCBA-504706', 'PCBA-504842',
                    'PCBA-504845', 'PCBA-504847', 'PCBA-504891', 'PCBA-540276', 'PCBA-540317',
                    'PCBA-588342', 'PCBA-588453', 'PCBA-588456', 'PCBA-588579', 'PCBA-588590',
                    'PCBA-588591', 'PCBA-588795', 'PCBA-588855', 'PCBA-602179', 'PCBA-602233',
                    'PCBA-602310', 'PCBA-602313', 'PCBA-602332', 'PCBA-624170', 'PCBA-624171',
                    'PCBA-624173', 'PCBA-624202', 'PCBA-624246', 'PCBA-624287', 'PCBA-624288',
                    'PCBA-624291', 'PCBA-624296', 'PCBA-624297', 'PCBA-624417', 'PCBA-651635',
                    'PCBA-651644', 'PCBA-651768', 'PCBA-651965', 'PCBA-652025', 'PCBA-652104',
                    'PCBA-652105', 'PCBA-652106', 'PCBA-686970', 'PCBA-686978', 'PCBA-686979',
                    'PCBA-720504', 'PCBA-720532', 'PCBA-720542', 'PCBA-720551', 'PCBA-720553',
                    'PCBA-720579', 'PCBA-720580', 'PCBA-720707', 'PCBA-720708', 'PCBA-720709',
                    'PCBA-720711', 'PCBA-743255', 'PCBA-743266', 'PCBA-875', 'PCBA-881',
                    'PCBA-883', 'PCBA-884', 'PCBA-885', 'PCBA-887', 'PCBA-891', 'PCBA-899',
                    'PCBA-902', 'PCBA-903', 'PCBA-904', 'PCBA-912', 'PCBA-914', 'PCBA-915',
                    'PCBA-924', 'PCBA-925', 'PCBA-926', 'PCBA-927', 'PCBA-938', 'PCBA-995'],
                   'classification', 'pcba', ['pcba.csv'], 'random', 'PRC-AUC'),
    'MUV': Config('https://s3-us-west-1.amazonaws.com/deepchem.io/datasets/molnet_publish/muv.zip',
                  'f6b813fd4012abd33680df12219d614f',
                  'smiles',
                  ['MUV-692', 'MUV-689', 'MUV-846', 'MUV-859', 'MUV-644', 'MUV-548',
                   'MUV-852', 'MUV-600', 'MUV-810', 'MUV-712', 'MUV-737', 'MUV-858',
                   'MUV-713', 'MUV-733', 'MUV-652', 'MUV-466', 'MUV-832'],
                  'classification', 'muv', ['muv.csv'], 'random', 'PRC-AUC'),
    'HIV': Config('https://s3-us-west-1.amazonaws.com/deepchem.io/datasets/molnet_publish/hiv.zip',
                  '60fe99f9ffeda3dd45ae5ce6e8416ecd',
                  'smiles',
                  ['HIV_active'],
                  'classification', 'hiv', ['HIV.csv'], 'scaffold', 'ROC-AUC'),
    # TODO: add PDBbind later, its too large, 1.91GB
    # 'PDBbind': Config('http://deepchem.io.s3-website-us-west-1.amazonaws.com/datasets/pdbbind_v2015.tar.gz',
    #                   '', '', []),
    'BACE': Config('https://s3-us-west-1.amazonaws.com/deepchem.io/datasets/molnet_publish/bace.zip',
                     '06ee55ddfdc84c4ae02a24ad676bdde8',
                     'mol',
                     ['Class'],
                     'classification', 'bace', ['bace.csv'], 'scaffold', 'ROC-AUC'),
    'BBBP': Config('https://s3-us-west-1.amazonaws.com/deepchem.io/datasets/molnet_publish/bbbp.zip',
                   'b786ba8001639a8aa9eaa5fd96d710d7',
                   'smiles',
                   ['p_np'],
                   'classification', 'bbbp', ['BBBP.csv'], 'scaffold', 'ROC-AUC'),
    'Tox21': Config('https://s3-us-west-1.amazonaws.com/deepchem.io/datasets/molnet_publish/tox21.zip',
                    '729b1079aab71ba703cb676050e53bcd',
                    'smiles',
                    ['NR-AR', 'NR-AR-LBD', 'NR-AhR', 'NR-Aromatase', 'NR-ER', 'NR-ER-LBD',
                     'NR-PPAR-gamma', 'SR-ARE', 'SR-ATAD5', 'SR-HSE', 'SR-MMP', 'SR-p53'],
                    'classification', 'tox21', ['tox21.csv'], 'random', 'ROC-AUC'),
    'ToxCast': Config('https://s3-us-west-1.amazonaws.com/deepchem.io/datasets/molnet_publish/toxcast.zip',
                      '89316c49db45511f5f9a2440a7cff0bf',
                      'smiles',
                      ['ACEA_T47D_80hr_Negative', 'ACEA_T47D_80hr_Positive', 'APR_HepG2_CellCycleArrest_24h_dn',
                       'APR_HepG2_CellCycleArrest_24h_up', 'APR_HepG2_CellCycleArrest_72h_dn',
                       'APR_HepG2_CellLoss_24h_dn', 'APR_HepG2_CellLoss_72h_dn', 'APR_HepG2_MicrotubuleCSK_24h_dn',
                       'APR_HepG2_MicrotubuleCSK_24h_up', 'APR_HepG2_MicrotubuleCSK_72h_dn',
                       'APR_HepG2_MicrotubuleCSK_72h_up', 'APR_HepG2_MitoMass_24h_dn', 'APR_HepG2_MitoMass_24h_up',
                       'APR_HepG2_MitoMass_72h_dn', 'APR_HepG2_MitoMass_72h_up', 'APR_HepG2_MitoMembPot_1h_dn',
                       'APR_HepG2_MitoMembPot_24h_dn', 'APR_HepG2_MitoMembPot_72h_dn', 'APR_HepG2_MitoticArrest_24h_up',
                       'APR_HepG2_MitoticArrest_72h_up', 'APR_HepG2_NuclearSize_24h_dn', 'APR_HepG2_NuclearSize_72h_dn',
                       'APR_HepG2_NuclearSize_72h_up', 'APR_HepG2_OxidativeStress_24h_up',
                       'APR_HepG2_OxidativeStress_72h_up', 'APR_HepG2_StressKinase_1h_up',
                       'APR_HepG2_StressKinase_24h_up', 'APR_HepG2_StressKinase_72h_up', 'APR_HepG2_p53Act_24h_up',
                       'APR_HepG2_p53Act_72h_up', 'APR_Hepat_Apoptosis_24hr_up', 'APR_Hepat_Apoptosis_48hr_up',
                       'APR_Hepat_CellLoss_24hr_dn', 'APR_Hepat_CellLoss_48hr_dn', 'APR_Hepat_DNADamage_24hr_up',
                       'APR_Hepat_DNADamage_48hr_up', 'APR_Hepat_DNATexture_24hr_up', 'APR_Hepat_DNATexture_48hr_up',
                       'APR_Hepat_MitoFxnI_1hr_dn', 'APR_Hepat_MitoFxnI_24hr_dn', 'APR_Hepat_MitoFxnI_48hr_dn',
                       'APR_Hepat_NuclearSize_24hr_dn', 'APR_Hepat_NuclearSize_48hr_dn', 'APR_Hepat_Steatosis_24hr_up',
                       'APR_Hepat_Steatosis_48hr_up', 'ATG_AP_1_CIS_dn', 'ATG_AP_1_CIS_up', 'ATG_AP_2_CIS_dn',
                       'ATG_AP_2_CIS_up', 'ATG_AR_TRANS_dn', 'ATG_AR_TRANS_up', 'ATG_Ahr_CIS_dn', 'ATG_Ahr_CIS_up',
                       'ATG_BRE_CIS_dn', 'ATG_BRE_CIS_up', 'ATG_CAR_TRANS_dn', 'ATG_CAR_TRANS_up', 'ATG_CMV_CIS_dn',
                       'ATG_CMV_CIS_up', 'ATG_CRE_CIS_dn', 'ATG_CRE_CIS_up', 'ATG_C_EBP_CIS_dn', 'ATG_C_EBP_CIS_up',
                       'ATG_DR4_LXR_CIS_dn', 'ATG_DR4_LXR_CIS_up', 'ATG_DR5_CIS_dn', 'ATG_DR5_CIS_up', 'ATG_E2F_CIS_dn',
                       'ATG_E2F_CIS_up', 'ATG_EGR_CIS_up', 'ATG_ERE_CIS_dn', 'ATG_ERE_CIS_up', 'ATG_ERRa_TRANS_dn',
                       'ATG_ERRg_TRANS_dn', 'ATG_ERRg_TRANS_up', 'ATG_ERa_TRANS_up', 'ATG_E_Box_CIS_dn',
                       'ATG_E_Box_CIS_up', 'ATG_Ets_CIS_dn', 'ATG_Ets_CIS_up', 'ATG_FXR_TRANS_up', 'ATG_FoxA2_CIS_dn',
                       'ATG_FoxA2_CIS_up', 'ATG_FoxO_CIS_dn', 'ATG_FoxO_CIS_up', 'ATG_GAL4_TRANS_dn', 'ATG_GATA_CIS_dn',
                       'ATG_GATA_CIS_up', 'ATG_GLI_CIS_dn', 'ATG_GLI_CIS_up', 'ATG_GRE_CIS_dn', 'ATG_GRE_CIS_up',
                       'ATG_GR_TRANS_dn', 'ATG_GR_TRANS_up', 'ATG_HIF1a_CIS_dn', 'ATG_HIF1a_CIS_up',
                       'ATG_HNF4a_TRANS_dn', 'ATG_HNF4a_TRANS_up', 'ATG_HNF6_CIS_dn', 'ATG_HNF6_CIS_up',
                       'ATG_HSE_CIS_dn', 'ATG_HSE_CIS_up', 'ATG_IR1_CIS_dn', 'ATG_IR1_CIS_up', 'ATG_ISRE_CIS_dn',
                       'ATG_ISRE_CIS_up', 'ATG_LXRa_TRANS_dn', 'ATG_LXRa_TRANS_up', 'ATG_LXRb_TRANS_dn',
                       'ATG_LXRb_TRANS_up', 'ATG_MRE_CIS_up', 'ATG_M_06_TRANS_up', 'ATG_M_19_CIS_dn',
                       'ATG_M_19_TRANS_dn', 'ATG_M_19_TRANS_up', 'ATG_M_32_CIS_dn', 'ATG_M_32_CIS_up',
                       'ATG_M_32_TRANS_dn', 'ATG_M_32_TRANS_up', 'ATG_M_61_TRANS_up', 'ATG_Myb_CIS_dn',
                       'ATG_Myb_CIS_up', 'ATG_Myc_CIS_dn', 'ATG_Myc_CIS_up', 'ATG_NFI_CIS_dn', 'ATG_NFI_CIS_up',
                       'ATG_NF_kB_CIS_dn', 'ATG_NF_kB_CIS_up', 'ATG_NRF1_CIS_dn', 'ATG_NRF1_CIS_up',
                       'ATG_NRF2_ARE_CIS_dn', 'ATG_NRF2_ARE_CIS_up', 'ATG_NURR1_TRANS_dn', 'ATG_NURR1_TRANS_up',
                       'ATG_Oct_MLP_CIS_dn', 'ATG_Oct_MLP_CIS_up', 'ATG_PBREM_CIS_dn', 'ATG_PBREM_CIS_up',
                       'ATG_PPARa_TRANS_dn', 'ATG_PPARa_TRANS_up', 'ATG_PPARd_TRANS_up', 'ATG_PPARg_TRANS_up',
                       'ATG_PPRE_CIS_dn', 'ATG_PPRE_CIS_up', 'ATG_PXRE_CIS_dn', 'ATG_PXRE_CIS_up', 'ATG_PXR_TRANS_dn',
                       'ATG_PXR_TRANS_up', 'ATG_Pax6_CIS_up', 'ATG_RARa_TRANS_dn', 'ATG_RARa_TRANS_up',
                       'ATG_RARb_TRANS_dn', 'ATG_RARb_TRANS_up', 'ATG_RARg_TRANS_dn', 'ATG_RARg_TRANS_up',
                       'ATG_RORE_CIS_dn', 'ATG_RORE_CIS_up', 'ATG_RORb_TRANS_dn', 'ATG_RORg_TRANS_dn',
                       'ATG_RORg_TRANS_up', 'ATG_RXRa_TRANS_dn', 'ATG_RXRa_TRANS_up', 'ATG_RXRb_TRANS_dn',
                       'ATG_RXRb_TRANS_up', 'ATG_SREBP_CIS_dn', 'ATG_SREBP_CIS_up', 'ATG_STAT3_CIS_dn',
                       'ATG_STAT3_CIS_up', 'ATG_Sox_CIS_dn', 'ATG_Sox_CIS_up', 'ATG_Sp1_CIS_dn', 'ATG_Sp1_CIS_up',
                       'ATG_TAL_CIS_dn', 'ATG_TAL_CIS_up', 'ATG_TA_CIS_dn', 'ATG_TA_CIS_up', 'ATG_TCF_b_cat_CIS_dn',
                       'ATG_TCF_b_cat_CIS_up', 'ATG_TGFb_CIS_dn', 'ATG_TGFb_CIS_up', 'ATG_THRa1_TRANS_dn',
                       'ATG_THRa1_TRANS_up', 'ATG_VDRE_CIS_dn', 'ATG_VDRE_CIS_up', 'ATG_VDR_TRANS_dn',
                       'ATG_VDR_TRANS_up', 'ATG_XTT_Cytotoxicity_up', 'ATG_Xbp1_CIS_dn', 'ATG_Xbp1_CIS_up',
                       'ATG_p53_CIS_dn', 'ATG_p53_CIS_up', 'BSK_3C_Eselectin_down', 'BSK_3C_HLADR_down',
                       'BSK_3C_ICAM1_down', 'BSK_3C_IL8_down', 'BSK_3C_MCP1_down', 'BSK_3C_MIG_down',
                       'BSK_3C_Proliferation_down', 'BSK_3C_SRB_down', 'BSK_3C_Thrombomodulin_down',
                       'BSK_3C_Thrombomodulin_up', 'BSK_3C_TissueFactor_down', 'BSK_3C_TissueFactor_up',
                       'BSK_3C_VCAM1_down', 'BSK_3C_Vis_down', 'BSK_3C_uPAR_down', 'BSK_4H_Eotaxin3_down',
                       'BSK_4H_MCP1_down', 'BSK_4H_Pselectin_down', 'BSK_4H_Pselectin_up', 'BSK_4H_SRB_down',
                       'BSK_4H_VCAM1_down', 'BSK_4H_VEGFRII_down', 'BSK_4H_uPAR_down', 'BSK_4H_uPAR_up',
                       'BSK_BE3C_HLADR_down', 'BSK_BE3C_IL1a_down', 'BSK_BE3C_IP10_down', 'BSK_BE3C_MIG_down',
                       'BSK_BE3C_MMP1_down', 'BSK_BE3C_MMP1_up', 'BSK_BE3C_PAI1_down', 'BSK_BE3C_SRB_down',
                       'BSK_BE3C_TGFb1_down', 'BSK_BE3C_tPA_down', 'BSK_BE3C_uPAR_down', 'BSK_BE3C_uPAR_up',
                       'BSK_BE3C_uPA_down', 'BSK_CASM3C_HLADR_down', 'BSK_CASM3C_IL6_down', 'BSK_CASM3C_IL6_up',
                       'BSK_CASM3C_IL8_down', 'BSK_CASM3C_LDLR_down', 'BSK_CASM3C_LDLR_up', 'BSK_CASM3C_MCP1_down',
                       'BSK_CASM3C_MCP1_up', 'BSK_CASM3C_MCSF_down', 'BSK_CASM3C_MCSF_up', 'BSK_CASM3C_MIG_down',
                       'BSK_CASM3C_Proliferation_down', 'BSK_CASM3C_Proliferation_up', 'BSK_CASM3C_SAA_down',
                       'BSK_CASM3C_SAA_up', 'BSK_CASM3C_SRB_down', 'BSK_CASM3C_Thrombomodulin_down',
                       'BSK_CASM3C_Thrombomodulin_up', 'BSK_CASM3C_TissueFactor_down', 'BSK_CASM3C_VCAM1_down',
                       'BSK_CASM3C_VCAM1_up', 'BSK_CASM3C_uPAR_down', 'BSK_CASM3C_uPAR_up', 'BSK_KF3CT_ICAM1_down',
                       'BSK_KF3CT_IL1a_down', 'BSK_KF3CT_IP10_down', 'BSK_KF3CT_IP10_up', 'BSK_KF3CT_MCP1_down',
                       'BSK_KF3CT_MCP1_up', 'BSK_KF3CT_MMP9_down', 'BSK_KF3CT_SRB_down', 'BSK_KF3CT_TGFb1_down',
                       'BSK_KF3CT_TIMP2_down', 'BSK_KF3CT_uPA_down', 'BSK_LPS_CD40_down', 'BSK_LPS_Eselectin_down',
                       'BSK_LPS_Eselectin_up', 'BSK_LPS_IL1a_down', 'BSK_LPS_IL1a_up', 'BSK_LPS_IL8_down',
                       'BSK_LPS_IL8_up', 'BSK_LPS_MCP1_down', 'BSK_LPS_MCSF_down', 'BSK_LPS_PGE2_down',
                       'BSK_LPS_PGE2_up', 'BSK_LPS_SRB_down', 'BSK_LPS_TNFa_down', 'BSK_LPS_TNFa_up',
                       'BSK_LPS_TissueFactor_down', 'BSK_LPS_TissueFactor_up', 'BSK_LPS_VCAM1_down',
                       'BSK_SAg_CD38_down', 'BSK_SAg_CD40_down', 'BSK_SAg_CD69_down', 'BSK_SAg_Eselectin_down',
                       'BSK_SAg_Eselectin_up', 'BSK_SAg_IL8_down', 'BSK_SAg_IL8_up', 'BSK_SAg_MCP1_down',
                       'BSK_SAg_MIG_down', 'BSK_SAg_PBMCCytotoxicity_down', 'BSK_SAg_PBMCCytotoxicity_up',
                       'BSK_SAg_Proliferation_down', 'BSK_SAg_SRB_down', 'BSK_hDFCGF_CollagenIII_down',
                       'BSK_hDFCGF_EGFR_down', 'BSK_hDFCGF_EGFR_up', 'BSK_hDFCGF_IL8_down', 'BSK_hDFCGF_IP10_down',
                       'BSK_hDFCGF_MCSF_down', 'BSK_hDFCGF_MIG_down', 'BSK_hDFCGF_MMP1_down', 'BSK_hDFCGF_MMP1_up',
                       'BSK_hDFCGF_PAI1_down', 'BSK_hDFCGF_Proliferation_down', 'BSK_hDFCGF_SRB_down',
                       'BSK_hDFCGF_TIMP1_down', 'BSK_hDFCGF_VCAM1_down', 'CEETOX_H295R_11DCORT_dn',
                       'CEETOX_H295R_ANDR_dn', 'CEETOX_H295R_CORTISOL_dn', 'CEETOX_H295R_DOC_dn', 'CEETOX_H295R_DOC_up',
                       'CEETOX_H295R_ESTRADIOL_dn', 'CEETOX_H295R_ESTRADIOL_up', 'CEETOX_H295R_ESTRONE_dn',
                       'CEETOX_H295R_ESTRONE_up', 'CEETOX_H295R_OHPREG_up', 'CEETOX_H295R_OHPROG_dn',
                       'CEETOX_H295R_OHPROG_up', 'CEETOX_H295R_PROG_up', 'CEETOX_H295R_TESTO_dn', 'CLD_ABCB1_48hr',
                       'CLD_ABCG2_48hr', 'CLD_CYP1A1_24hr', 'CLD_CYP1A1_48hr', 'CLD_CYP1A1_6hr', 'CLD_CYP1A2_24hr',
                       'CLD_CYP1A2_48hr', 'CLD_CYP1A2_6hr', 'CLD_CYP2B6_24hr', 'CLD_CYP2B6_48hr', 'CLD_CYP2B6_6hr',
                       'CLD_CYP3A4_24hr', 'CLD_CYP3A4_48hr', 'CLD_CYP3A4_6hr', 'CLD_GSTA2_48hr', 'CLD_SULT2A_24hr',
                       'CLD_SULT2A_48hr', 'CLD_UGT1A1_24hr', 'CLD_UGT1A1_48hr', 'NCCT_HEK293T_CellTiterGLO',
                       'NCCT_QuantiLum_inhib_2_dn', 'NCCT_QuantiLum_inhib_dn', 'NCCT_TPO_AUR_dn', 'NCCT_TPO_GUA_dn',
                       'NHEERL_ZF_144hpf_TERATOSCORE_up', 'NVS_ADME_hCYP19A1', 'NVS_ADME_hCYP1A1', 'NVS_ADME_hCYP1A2',
                       'NVS_ADME_hCYP2A6', 'NVS_ADME_hCYP2B6', 'NVS_ADME_hCYP2C19', 'NVS_ADME_hCYP2C9',
                       'NVS_ADME_hCYP2D6', 'NVS_ADME_hCYP3A4', 'NVS_ADME_hCYP4F12', 'NVS_ADME_rCYP2C12',
                       'NVS_ENZ_hAChE', 'NVS_ENZ_hAMPKa1', 'NVS_ENZ_hAurA', 'NVS_ENZ_hBACE', 'NVS_ENZ_hCASP5',
                       'NVS_ENZ_hCK1D', 'NVS_ENZ_hDUSP3', 'NVS_ENZ_hES', 'NVS_ENZ_hElastase', 'NVS_ENZ_hFGFR1',
                       'NVS_ENZ_hGSK3b', 'NVS_ENZ_hMMP1', 'NVS_ENZ_hMMP13', 'NVS_ENZ_hMMP2', 'NVS_ENZ_hMMP3',
                       'NVS_ENZ_hMMP7', 'NVS_ENZ_hMMP9', 'NVS_ENZ_hPDE10', 'NVS_ENZ_hPDE4A1', 'NVS_ENZ_hPDE5',
                       'NVS_ENZ_hPI3Ka', 'NVS_ENZ_hPTEN', 'NVS_ENZ_hPTPN11', 'NVS_ENZ_hPTPN12', 'NVS_ENZ_hPTPN13',
                       'NVS_ENZ_hPTPN9', 'NVS_ENZ_hPTPRC', 'NVS_ENZ_hSIRT1', 'NVS_ENZ_hSIRT2', 'NVS_ENZ_hTrkA',
                       'NVS_ENZ_hVEGFR2', 'NVS_ENZ_oCOX1', 'NVS_ENZ_oCOX2', 'NVS_ENZ_rAChE', 'NVS_ENZ_rCNOS',
                       'NVS_ENZ_rMAOAC', 'NVS_ENZ_rMAOAP', 'NVS_ENZ_rMAOBC', 'NVS_ENZ_rMAOBP', 'NVS_ENZ_rabI2C',
                       'NVS_GPCR_bAdoR_NonSelective', 'NVS_GPCR_bDR_NonSelective', 'NVS_GPCR_g5HT4', 'NVS_GPCR_gH2',
                       'NVS_GPCR_gLTB4', 'NVS_GPCR_gLTD4', 'NVS_GPCR_gMPeripheral_NonSelective', 'NVS_GPCR_gOpiateK',
                       'NVS_GPCR_h5HT2A', 'NVS_GPCR_h5HT5A', 'NVS_GPCR_h5HT6', 'NVS_GPCR_h5HT7', 'NVS_GPCR_hAT1',
                       'NVS_GPCR_hAdoRA1', 'NVS_GPCR_hAdoRA2a', 'NVS_GPCR_hAdra2A', 'NVS_GPCR_hAdra2C',
                       'NVS_GPCR_hAdrb1', 'NVS_GPCR_hAdrb2', 'NVS_GPCR_hAdrb3', 'NVS_GPCR_hDRD1', 'NVS_GPCR_hDRD2s',
                       'NVS_GPCR_hDRD4.4', 'NVS_GPCR_hH1', 'NVS_GPCR_hLTB4_BLT1', 'NVS_GPCR_hM1', 'NVS_GPCR_hM2',
                       'NVS_GPCR_hM3', 'NVS_GPCR_hM4', 'NVS_GPCR_hNK2', 'NVS_GPCR_hOpiate_D1', 'NVS_GPCR_hOpiate_mu',
                       'NVS_GPCR_hTXA2', 'NVS_GPCR_p5HT2C', 'NVS_GPCR_r5HT1_NonSelective', 'NVS_GPCR_r5HT_NonSelective',
                       'NVS_GPCR_rAdra1B', 'NVS_GPCR_rAdra1_NonSelective', 'NVS_GPCR_rAdra2_NonSelective',
                       'NVS_GPCR_rAdrb_NonSelective', 'NVS_GPCR_rNK1', 'NVS_GPCR_rNK3', 'NVS_GPCR_rOpiate_NonSelective',
                       'NVS_GPCR_rOpiate_NonSelectiveNa', 'NVS_GPCR_rSST', 'NVS_GPCR_rTRH', 'NVS_GPCR_rV1',
                       'NVS_GPCR_rabPAF', 'NVS_GPCR_rmAdra2B', 'NVS_IC_hKhERGCh', 'NVS_IC_rCaBTZCHL',
                       'NVS_IC_rCaDHPRCh_L', 'NVS_IC_rNaCh_site2', 'NVS_LGIC_bGABARa1', 'NVS_LGIC_h5HT3',
                       'NVS_LGIC_hNNR_NBungSens', 'NVS_LGIC_rGABAR_NonSelective',
                       'NVS_LGIC_rNNR_BungSens', 'NVS_MP_hPBR', 'NVS_MP_rPBR', 'NVS_NR_bER', 'NVS_NR_bPR', 'NVS_NR_cAR',
                       'NVS_NR_hAR', 'NVS_NR_hCAR_Antagonist', 'NVS_NR_hER', 'NVS_NR_hFXR_Agonist',
                       'NVS_NR_hFXR_Antagonist', 'NVS_NR_hGR', 'NVS_NR_hPPARa', 'NVS_NR_hPPARg', 'NVS_NR_hPR',
                       'NVS_NR_hPXR', 'NVS_NR_hRAR_Antagonist', 'NVS_NR_hRARa_Agonist', 'NVS_NR_hTRa_Antagonist',
                       'NVS_NR_mERa', 'NVS_NR_rAR', 'NVS_NR_rMR', 'NVS_OR_gSIGMA_NonSelective', 'NVS_TR_gDAT',
                       'NVS_TR_hAdoT', 'NVS_TR_hDAT', 'NVS_TR_hNET', 'NVS_TR_hSERT', 'NVS_TR_rNET', 'NVS_TR_rSERT',
                       'NVS_TR_rVMAT2', 'OT_AR_ARELUC_AG_1440', 'OT_AR_ARSRC1_0480', 'OT_AR_ARSRC1_0960',
                       'OT_ER_ERaERa_0480', 'OT_ER_ERaERa_1440', 'OT_ER_ERaERb_0480', 'OT_ER_ERaERb_1440',
                       'OT_ER_ERbERb_0480', 'OT_ER_ERbERb_1440', 'OT_ERa_EREGFP_0120', 'OT_ERa_EREGFP_0480',
                       'OT_FXR_FXRSRC1_0480', 'OT_FXR_FXRSRC1_1440', 'OT_NURR1_NURR1RXRa_0480',
                       'OT_NURR1_NURR1RXRa_1440', 'TOX21_ARE_BLA_Agonist_ch1', 'TOX21_ARE_BLA_Agonist_ch2',
                       'TOX21_ARE_BLA_agonist_ratio', 'TOX21_ARE_BLA_agonist_viability', 'TOX21_AR_BLA_Agonist_ch1',
                       'TOX21_AR_BLA_Agonist_ch2', 'TOX21_AR_BLA_Agonist_ratio', 'TOX21_AR_BLA_Antagonist_ch1',
                       'TOX21_AR_BLA_Antagonist_ch2', 'TOX21_AR_BLA_Antagonist_ratio',
                       'TOX21_AR_BLA_Antagonist_viability', 'TOX21_AR_LUC_MDAKB2_Agonist',
                       'TOX21_AR_LUC_MDAKB2_Antagonist', 'TOX21_AR_LUC_MDAKB2_Antagonist2', 'TOX21_AhR_LUC_Agonist',
                       'TOX21_Aromatase_Inhibition', 'TOX21_AutoFluor_HEK293_Cell_blue',
                       'TOX21_AutoFluor_HEK293_Media_blue', 'TOX21_AutoFluor_HEPG2_Cell_blue',
                       'TOX21_AutoFluor_HEPG2_Cell_green', 'TOX21_AutoFluor_HEPG2_Media_blue',
                       'TOX21_AutoFluor_HEPG2_Media_green', 'TOX21_ELG1_LUC_Agonist', 'TOX21_ERa_BLA_Agonist_ch1',
                       'TOX21_ERa_BLA_Agonist_ch2', 'TOX21_ERa_BLA_Agonist_ratio', 'TOX21_ERa_BLA_Antagonist_ch1',
                       'TOX21_ERa_BLA_Antagonist_ch2', 'TOX21_ERa_BLA_Antagonist_ratio',
                       'TOX21_ERa_BLA_Antagonist_viability', 'TOX21_ERa_LUC_BG1_Agonist',
                       'TOX21_ERa_LUC_BG1_Antagonist', 'TOX21_ESRE_BLA_ch1', 'TOX21_ESRE_BLA_ch2',
                       'TOX21_ESRE_BLA_ratio', 'TOX21_ESRE_BLA_viability',
                       'TOX21_FXR_BLA_Antagonist_ch1', 'TOX21_FXR_BLA_Antagonist_ch2', 'TOX21_FXR_BLA_agonist_ch2',
                       'TOX21_FXR_BLA_agonist_ratio', 'TOX21_FXR_BLA_antagonist_ratio',
                       'TOX21_FXR_BLA_antagonist_viability', 'TOX21_GR_BLA_Agonist_ch1', 'TOX21_GR_BLA_Agonist_ch2',
                       'TOX21_GR_BLA_Agonist_ratio', 'TOX21_GR_BLA_Antagonist_ch2', 'TOX21_GR_BLA_Antagonist_ratio',
                       'TOX21_GR_BLA_Antagonist_viability', 'TOX21_HSE_BLA_agonist_ch1', 'TOX21_HSE_BLA_agonist_ch2',
                       'TOX21_HSE_BLA_agonist_ratio', 'TOX21_HSE_BLA_agonist_viability', 'TOX21_MMP_ratio_down',
                       'TOX21_MMP_ratio_up', 'TOX21_MMP_viability', 'TOX21_NFkB_BLA_agonist_ch1',
                       'TOX21_NFkB_BLA_agonist_ch2', 'TOX21_NFkB_BLA_agonist_ratio', 'TOX21_NFkB_BLA_agonist_viability',
                       'TOX21_PPARd_BLA_Agonist_viability', 'TOX21_PPARd_BLA_Antagonist_ch1',
                       'TOX21_PPARd_BLA_agonist_ch1', 'TOX21_PPARd_BLA_agonist_ch2', 'TOX21_PPARd_BLA_agonist_ratio',
                       'TOX21_PPARd_BLA_antagonist_ratio', 'TOX21_PPARd_BLA_antagonist_viability',
                       'TOX21_PPARg_BLA_Agonist_ch1', 'TOX21_PPARg_BLA_Agonist_ch2', 'TOX21_PPARg_BLA_Agonist_ratio',
                       'TOX21_PPARg_BLA_Antagonist_ch1', 'TOX21_PPARg_BLA_antagonist_ratio',
                       'TOX21_PPARg_BLA_antagonist_viability',
                       'TOX21_TR_LUC_GH3_Agonist', 'TOX21_TR_LUC_GH3_Antagonist', 'TOX21_VDR_BLA_Agonist_viability',
                       'TOX21_VDR_BLA_Antagonist_ch1', 'TOX21_VDR_BLA_agonist_ch2', 'TOX21_VDR_BLA_agonist_ratio',
                       'TOX21_VDR_BLA_antagonist_ratio', 'TOX21_VDR_BLA_antagonist_viability', 'TOX21_p53_BLA_p1_ch1',
                       'TOX21_p53_BLA_p1_ch2', 'TOX21_p53_BLA_p1_ratio', 'TOX21_p53_BLA_p1_viability',
                       'TOX21_p53_BLA_p2_ch1', 'TOX21_p53_BLA_p2_ch2', 'TOX21_p53_BLA_p2_ratio',
                       'TOX21_p53_BLA_p2_viability', 'TOX21_p53_BLA_p3_ch1', 'TOX21_p53_BLA_p3_ch2',
                       'TOX21_p53_BLA_p3_ratio', 'TOX21_p53_BLA_p3_viability',
                       'TOX21_p53_BLA_p4_ch1', 'TOX21_p53_BLA_p4_ch2', 'TOX21_p53_BLA_p4_ratio',
                       'TOX21_p53_BLA_p4_viability', 'TOX21_p53_BLA_p5_ch1', 'TOX21_p53_BLA_p5_ch2',
                       'TOX21_p53_BLA_p5_ratio', 'TOX21_p53_BLA_p5_viability', 'Tanguay_ZF_120hpf_AXIS_up',
                       'Tanguay_ZF_120hpf_ActivityScore', 'Tanguay_ZF_120hpf_BRAI_up', 'Tanguay_ZF_120hpf_CFIN_up',
                       'Tanguay_ZF_120hpf_CIRC_up', 'Tanguay_ZF_120hpf_EYE_up', 'Tanguay_ZF_120hpf_JAW_up',
                       'Tanguay_ZF_120hpf_MORT_up', 'Tanguay_ZF_120hpf_OTIC_up', 'Tanguay_ZF_120hpf_PE_up',
                       'Tanguay_ZF_120hpf_PFIN_up', 'Tanguay_ZF_120hpf_PIG_up', 'Tanguay_ZF_120hpf_SNOU_up',
                       'Tanguay_ZF_120hpf_SOMI_up', 'Tanguay_ZF_120hpf_SWIM_up', 'Tanguay_ZF_120hpf_TRUN_up',
                       'Tanguay_ZF_120hpf_TR_up', 'Tanguay_ZF_120hpf_YSE_up'],
                      'classification', 'toxcast', ['toxcast_data.csv'], 'random', 'ROC-AUC'),
    'SIDER': Config('https://s3-us-west-1.amazonaws.com/deepchem.io/datasets/molnet_publish/sider.zip',
                    'eda68515eaddac9038e6302ce69966ef',
                    'smiles',
                    ['Hepatobiliary disorders', 'Metabolism and nutrition disorders', 'Product issues', 'Eye disorders',
                     'Investigations', 'Musculoskeletal and connective tissue disorders', 'Gastrointestinal disorders',
                     'Social circumstances', 'Immune system disorders', 'Reproductive system and breast disorders',
                     'Neoplasms benign, malignant and unspecified (incl cysts and polyps)',
                     'General disorders and administration site conditions', 'Endocrine disorders',
                     'Surgical and medical procedures', 'Vascular disorders', 'Blood and lymphatic system disorders',
                     'Skin and subcutaneous tissue disorders', 'Congenital, familial and genetic disorders',
                     'Infections and infestations', 'Respiratory, thoracic and mediastinal disorders',
                     'Psychiatric disorders', 'Renal and urinary disorders',
                     'Pregnancy, puerperium and perinatal conditions', 'Ear and labyrinth disorders',
                     'Cardiac disorders', 'Nervous system disorders', 'Injury, poisoning and procedural complications'],
                    'classification', 'sider', ['sider.csv'], 'random', 'ROC-AUC'),
    'ClinTox': Config('https://s3-us-west-1.amazonaws.com/deepchem.io/datasets/molnet_publish/clintox.zip',
                      '766001b0d3677a62b498fdcfe2e76ba3',
                      'smiles',
                      ['FDA_APPROVED', 'CT_TOX'],
                      'classification', 'clintox', ['clintox.csv'], 'random', 'ROC-AUC'),
}




import pandas as pd
import numpy as np
from pathlib import Path
from rdkit import RDLogger
from rdkit import Chem
import tqdm
import cloudpickle

from molnet.molnet_config import molnet_config
from molnet.download import download

lg = RDLogger.logger()
lg.setLevel(RDLogger.CRITICAL)


class MolDataset:
    def __init__(self, mols, y):
        """
        Numpy molecule dataset
        :param mols: N molecules
        :param y: N * num_tasks matrix
        """
        self.mols = np.asarray(mols)
        self.y = np.asarray(y)
        if len(self.y.shape) == 1:
            self.y = np.expand_dims(self.y, axis=1)  # make this shape (N, 1)
        assert len(self.y.shape) == 2
        self.w = None
        self._filter(based_on=self.mols, what=None)
        assert len(self.mols) == len(self.y)

    def __len__(self):
        return len(self.mols)

    def __repr__(self):
        return f'<MolDataset: {len(self)} molecules.>'

    def _filter(self, based_on, what):
        mask = (based_on != what)
        self.mols = self.mols[mask]
        self.y = self.y[mask]
        if self.w:
            self.w = self.w[mask]

    def calc_weight_classification(self):
        mask = np.isnan(self.y)
        assert np.all(np.unique(self.y[~mask]) == np.array([0,1]))
        self.w = np.ones_like(self.y, dtype=float)
        # target value is nan, then corresponding weight is 0
        self.w[mask] = 0
        for col in range(self.y.shape[1]):
            y, w = self.y[:, col], self.w[:, col]
            pos_mask = ~mask[:,col] & (y == 1)
            neg_mask = ~mask[:,col] & (y == 0)
            pos_num = pos_mask.sum()
            neg_num = neg_mask.sum()
            # negative weight is 1
            pos_w = neg_num / pos_num
            self.w[pos_mask, col] = pos_w
            self.w[neg_mask, col] = 1

    def cal_weight_basic(self):
        mask = np.isnan(self.y)
        self.w = np.ones_like(self.y, dtype=float)
        # target value is nan, then corresponding weight is 0
        self.w[mask] = 0

    def randomize(self):
        index = np.random.permutation(len(self.mols))
        self.mols = self.mols[index]
        self.y = self.y[index]
        if self.w:
            self.w = self.w[index]

    def subset(self, idx):
        mols = self.mols[idx]
        y = self.y[idx]
        return MolDataset(mols, y)

    @staticmethod
    def merge(dataset_lst):
        mols = np.concatenate([d.mols for d in dataset_lst], axis=0)
        y = np.concatenate([d.y for d in dataset_lst], axis=0)
        return MolDataset(mols, y)


def load_QM7(datadir, save=False):
    prefix = Path(datadir) / molnet_config['QM7'].relative_path
    download('QM7', datadir)
    cache_file = prefix / ('QM7' + '.bin')
    if cache_file.is_file():
        dataset = cloudpickle.load(open(cache_file, 'rb'))
        return dataset
    df = pd.read_csv(prefix / molnet_config['QM7'].files[0])
    mols = np.array([Chem.MolFromSmiles(i) for i in tqdm.tqdm(df['smiles'])])
    dataset = MolDataset(mols, df['u0_atom'].to_numpy())
    if save:
        cloudpickle.dump(dataset, open(cache_file, 'wb'))
    return dataset


# def load_QM7b(datadir, save=False):
#     prefix = Path(datadir) / molnet_config['QM7b'].relative_path
#     download('QM7b', datadir)
#     cache_file = prefix / ('QM7b' + '.bin')
#     if cache_file.is_file():
#         dataset = cloudpickle.load(open(cache_file, 'rb'))
#         return dataset
#     df = pd.read_csv(prefix / molnet_config['QM7b'].files[0])
#     mols = np.array([Chem.MolFromSmiles(i) for i in tqdm.tqdm(df['smiles'])])
#     dataset = MolDataset(mols, df['u0_atom'].to_numpy())
#     if save:
#         cloudpickle.dump(dataset, open(cache_file, 'wb'))
#     return dataset


def load_QM8(datadir, save=False):
    prefix = Path(datadir) / molnet_config['QM8'].relative_path
    download('QM8', datadir)
    cache_file = prefix / ('QM8' + '.bin')
    if cache_file.is_file():
        dataset = cloudpickle.load(open(cache_file, 'rb'))
        return dataset
    df = pd.read_csv(prefix / molnet_config['QM8'].files[0])
    # mols = df[molnet_config['QM8'].smiles_col].to_numpy()
    suppl = Chem.SDMolSupplier(str(prefix / molnet_config['QM8'].files[1]))
    mols = np.array([m for m in tqdm.tqdm(suppl)])
    y = df[molnet_config['QM8'].tasks_lst].to_numpy()
    dataset = MolDataset(mols, y)
    if save:
        cloudpickle.dump(dataset, open(cache_file, 'wb'))
    return dataset


def load_QM9(datadir, save=False):
    prefix = Path(datadir) / molnet_config['QM9'].relative_path
    download('QM9', datadir)
    cache_file = prefix / ('QM9' + '.bin')
    if cache_file.is_file():
        dataset = cloudpickle.load(open(cache_file, 'rb'))
        return dataset
    df = pd.read_csv(prefix / molnet_config['QM9'].files[0])
    # mols = df[molnet_config['QM8'].smiles_col].to_numpy()
    suppl = Chem.SDMolSupplier(str(prefix / molnet_config['QM9'].files[1]))
    mols = np.array([m for m in tqdm.tqdm(suppl)])
    y = df[molnet_config['QM9'].tasks_lst].to_numpy()
    dataset = MolDataset(mols, y)
    if save:
        cloudpickle.dump(dataset, open(cache_file, 'wb'))
    return dataset


def load_single_csv(name):
    def loadfn(datadir, save=False):
        prefix = Path(datadir) / molnet_config[name].relative_path
        download(name, datadir)
        cache_file = prefix / (name + '.bin')
        if cache_file.is_file():
            dataset = cloudpickle.load(open(cache_file, 'rb'))
            return dataset
        df = pd.read_csv(prefix / molnet_config[name].files[0])
        mols = np.array([Chem.MolFromSmiles(smi) for smi in tqdm.tqdm(df[molnet_config[name].smiles_col])])
        y = df[molnet_config[name].tasks_lst].to_numpy()
        dataset = MolDataset(mols, y)
        if save:
            cloudpickle.dump(dataset, open(cache_file, 'wb'))
        return dataset
    return loadfn


load_ESOL = load_single_csv('ESOL')
load_FreeSolv = load_single_csv('FreeSolv')
load_Lipophilicity = load_single_csv('Lipophilicity')
load_PCBA = load_single_csv('PCBA')
load_MUV = load_single_csv('MUV')
load_HIV = load_single_csv('HIV')
load_BACE = load_single_csv('BACE')
load_BBBP = load_single_csv('BBBP')
load_Tox21 = load_single_csv('Tox21')
load_ToxCast = load_single_csv('ToxCast')
load_SIDER = load_single_csv('SIDER')
load_ClinTox = load_single_csv('ClinTox')

if __name__ == '__main__':
    print(load_QM9('../'))



import requests
import tqdm
from pathlib import Path
import zipfile
import sys

from molnet.utils import md5
from molnet.molnet_config import molnet_config


def download(dataset, dest='/tmp'):
    task_config = molnet_config[dataset]
    fpath = Path(dest) / task_config.fname
    if not (fpath.is_file() and md5(fpath) == task_config.hash):
        r = requests.get(task_config.url, stream=True)
        total_size = int(r.headers.get('content-length', 0))
        block = 100  # 100KB
        with open(fpath, 'wb') as f:
            with tqdm.tqdm(total=total_size, unit='B', unit_scale=True, unit_divisor=1024) as pbar:
                for data in r.iter_content(block * 1024):
                    f.write(data)
                    pbar.update(len(data))
    else:
        print("Dataset file already downloaded.", file=sys.stderr)
    if fpath.suffix == '.zip':
        if not (fpath.parent / fpath.stem).is_dir():
            with zipfile.ZipFile(fpath, 'r') as zip_ref:
                zip_ref.extractall(fpath.parent / fpath.stem)
        else:
            print("Dataset file already unzipped", file=sys.stderr)
    return fpath





from rdkit.Chem import SaltRemover
from rdkit.Chem import Descriptors
import hashlib
from rdkit import Chem
import os


class Config:
    def __init__(self, url, hash, smiles_col, tasks_lst, task_type, relative_path, files, split, metric):
        self.url = url
        self.hash = hash
        self.smiles_col = smiles_col
        self.tasks_lst = tasks_lst
        self.task_type = task_type
        self.relative_path = relative_path
        # extracted file list
        self.files = files
        # directly downloaded file name
        self.fname = url.rsplit('/', 1)[-1]
        self.split = split
        self.metric = metric

        self.load_fn = None

    def add_load_fn(self, fn):
        self.load_fn = fn


def md5(fname):
    hash_md5 = hashlib.md5()
    with open(fname, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hash_md5.update(chunk)
    # this returns a string
    return hash_md5.hexdigest()


"""
Functions that can be used to preprocess SMILES sequnces in the form used in the publication.
This file is modified from https://github.com/jrwnter/cddd/blob/master/cddd/preprocessing.py
So that the training data are same. Please refer to this repo and their paper for further information.

Modified: minor RDKit function calls, docstring types.
"""

REMOVER = SaltRemover.SaltRemover()
ORGANIC_ATOM_SET = {5, 6, 7, 8, 9, 14, 15, 16, 17, 35, 53}
#                   B, C, N, O, F, Si, P,  S,  Cl, Br, I

# def canonical_smile(sml):
#     """
#     Helper Function that returns the RDKit canonical SMILES for a input SMILES sequnce
#     :param str sml: A SMILES sequence.
#     :return: A canonical SMILES sequence.
#     :rtype: str
#     """
#     # modified here to first transform `sml` to a `Mol` instance.
#     return Chem.MolToSmiles(Chem.MolFromSmiles(sml), canonical=True)





# def organic_filter(sml):
#     """
#     Function that filters for organic molecules.
#     :param str sml: A SMILES sequence.
#     :return: If `sml` can be interpreted by RDKit and is organic.
#     :rtype: bool
#     """
#     try:
#         m = Chem.MolFromSmiles(sml)
#         atom_num_list = [atom.GetAtomicNum() for atom in m.GetAtoms()]
#         is_organic = (set(atom_num_list) <= ORGANIC_ATOM_SET)
#         if is_organic:
#             return True
#         else:
#             return False
#     except:
#         return False


# def filter_smiles(sml):
#     """
#     Return the canonical SMILES sequence when fulfilled rules:
#         - -5 < LogP < 7
#         - 12 < mol_weight < 600
#         - 3 < num_heavy_atoms < 50
#         - is organic (in `ORGANIC_ATOM_SET`)
#     Return None Any input SMILES violate the rules will make this
#     :param str sml: A SMILES sequence.
#     :return: A canonical SMILES sequence or None.
#     :rtype: Union[str, NoneType]
#     """
#     try:
#         m = Chem.MolFromSmiles(sml)
#         logp = Descriptors.MolLogP(m)
#         mol_weight = Descriptors.MolWt(m)
#         num_heavy_atoms = Descriptors.HeavyAtomCount(m)
#         atom_num_list = [atom.GetAtomicNum() for atom in m.GetAtoms()]
#         is_organic = set(atom_num_list) <= ORGANIC_ATOM_SET
#         if ((logp > -5) & (logp < 7) &
#                 (mol_weight > 12) & (mol_weight < 600) &
#                 (num_heavy_atoms > 3) & (num_heavy_atoms < 50) &
#                 is_organic):
#             return Chem.MolToSmiles(m)
#         else:
#             return None
#     except:
#         return None


# def preprocess_smiles(sml):
#     """
#     Function that preprocesses a SMILES string such that it is in the same format as
#     the translation model was trained on. It removes salts from the
#     SMILES sequnce. If the sequnce correspond to an inorganic molecule or cannot be
#     interpreted by RDKit, None is returned.
#     :param str sml: A SMILES sequence.
#     :return: A canonical SMILES sequence or None.
#     :rtype: Union[str, NoneType]
#     """
#     new_sml = remove_salt(sml, REMOVER)
#     # new_sml = filter_smiles(new_sml)
#     return new_sml




from pathlib import Path
import sys
from rdkit.Chem.Scaffolds import MurckoScaffold
from rdkit import Chem
import numpy as np
import cloudpickle


def generate_scaffold(smiles, include_chirality=False):
    """Compute the Bemis-Murcko scaffold for a SMILES string."""
    if isinstance(smiles, Chem.Mol):
        mol = smiles
    else:
        mol = Chem.MolFromSmiles(smiles)
    scaffold = MurckoScaffold.MurckoScaffoldSmiles(mol=mol, includeChirality=include_chirality)
    return scaffold


class Splitter(object):
    def __init__(self, save_to=None):
        self.save_to = save_to

    def save_split(self, name, obj_lst):
        if self.save_to is not None:
            p = Path(self.save_to)
            cloudpickle.dump(obj_lst, open(p/name, 'wb'))
            return p/name
        return None

    def load_split(self, name):
        if self.save_to is None:
            return None
        p = Path(self.save_to)
        if not (p/name).is_file():
            return None
        print(f'Loading splits from cached file: {p/name}', file=sys.stderr)
        obj = cloudpickle.load(open(p/name, 'rb'))
        return obj

    def k_fold_split(self, dataset, k, seed=None):
        """
        Do k fold cv split
        :param dataset: whole dataset
        :param k: k fold
        :param seed: random seed
        :return: a list of subsets
        """
        cache = self.load_split('KFold.split')
        if cache is not None:
            return cache
        cv_datasets = []
        # rem_dataset is remaining portion of dataset
        rem_dataset = dataset
        for fold in range(k):
            # for example, k=5, a) select 1*1/(5-0)=1/5, remain 4/5; b) select 4/5*1/(5-1)=1/5, remain 3/5 ...
            frac_fold = 1. / (k - fold)
            fold_inds, rem_inds, _ = self.split(rem_dataset, frac_train=frac_fold,
                                                frac_valid=1-frac_fold, frac_test=0, seed=seed)
            cv_dataset = rem_dataset.subset(fold_inds)
            cv_datasets.append(cv_dataset)
            rem_dataset = rem_dataset.subset(rem_inds)
            self.save_split('KFold.split', cv_datasets)
        return cv_datasets

    def train_valid_test_split(self, dataset, frac_train=.8, frac_valid=.1, frac_test=.1, seed=None):
        """
        do train valid test split
        :param dataset:
        :param frac_train:
        :param frac_valid:
        :param frac_test:
        :param seed:
        :return:
        """
        cache = self.load_split('train_valid_test.split')
        if cache is not None:
            return cache
        train_inds, valid_inds, test_inds = self.split(dataset, frac_train=frac_train,
                                                       frac_test=frac_test, frac_valid=frac_valid, seed=seed)
        train_dataset = dataset.subset(train_inds)
        if len(valid_inds) != 0:
            valid_dataset = dataset.subset(valid_inds)
        else:
            valid_dataset = None
        if len(test_inds) != 0:
            test_dataset = dataset.subset(test_inds)
        else:
            test_dataset = None
        self.save_split('train_valid_test.split', (train_dataset, valid_dataset, test_dataset))
        return train_dataset, valid_dataset, test_dataset

    def train_test_split(self, dataset, frac_train=.8, seed=None):
        """
        Simple wraper of train_valid_test_split function
        :param dataset:
        :param frac_train:
        :param seed:
        :return:
        """
        cache = self.load_split('train_test.split')
        if cache is not None:
            return cache
        train_dataset, _, test_dataset = self.train_valid_test_split(dataset, frac_train=frac_train,
                                                                     frac_test=1 - frac_train, frac_valid=0,
                                                                     seed=seed)
        self.save_split('train_test.split', (train_dataset, test_dataset))
        return train_dataset, test_dataset

    def split(self, dataset, frac_train, frac_valid, frac_test, seed):
        raise NotImplementedError


class RandomSplitter(Splitter):
    """
    Class for random split molecules
    """
    def split(self, dataset, frac_train=.8, frac_valid=.1, frac_test=.1, seed=None):
        np.testing.assert_almost_equal(frac_train + frac_valid + frac_test, 1.)
        rng = np.random.RandomState(seed)
        perm = rng.permutation(len(dataset))
        train_cutoff = int(len(dataset) * frac_train)
        valid_cutoff = int(len(dataset) * (frac_train + frac_valid))
        return perm[:train_cutoff], perm[train_cutoff:valid_cutoff], perm[valid_cutoff:]


class ScaffoldSplitter(Splitter):
    """
    Class for doing data splits based on the scaffold of small molecules.
    """

    def split(self, dataset, frac_train=.8, frac_valid=.1, frac_test=.1, seed=None):
        """
        Do scaffold split
        :param dataset:
        :param frac_train:
        :param frac_valid:
        :param frac_test:
        :param seed: seed is useless here for scaffold split is not random
        :return:
        """
        np.testing.assert_almost_equal(frac_train + frac_valid + frac_test, 1.)
        if seed is not None:
            print("Scaffold split is not a random procedure, so seed is useless here", file=sys.stderr)
        scaffolds = {}
        for ind, smiles in enumerate(dataset.mols):
            scaffold = generate_scaffold(smiles)
            if scaffold not in scaffolds:
                scaffolds[scaffold] = [ind]
            else:
                scaffolds[scaffold].append(ind)
        # Sort from largest to smallest scaffold sets
        scaffolds = {key: sorted(value) for key, value in scaffolds.items()}
        # TODO: Why here is sorted? this will produce same train, valid and test set on every time!
        scaffold_sets = [scaffold_set for (scaffold, scaffold_set) in
                         sorted(scaffolds.items(), key=lambda x: (len(x[1]), x[1][0]), reverse=True)]
        train_cutoff = frac_train * len(dataset)
        valid_cutoff = (frac_train + frac_valid) * len(dataset)
        train_inds, valid_inds, test_inds = [], [], []
        for scaffold_set in scaffold_sets:
            if len(train_inds) + len(scaffold_set) > train_cutoff:
                if len(train_inds) + len(valid_inds) + len(scaffold_set) > valid_cutoff:
                    test_inds += scaffold_set
                else:
                    valid_inds += scaffold_set
            else:
                train_inds += scaffold_set
        return train_inds, valid_inds, test_inds



import setuptools
import molnet

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="molnet-python", # Replace with your own username
    version=molnet.__version__,
    author="Minys",
    license='MIT',
    author_email="minys@foxmail.com",
    description="A small package for loading data from MoleculeNet",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Minys233/molnet-python.git",
    keywords=['MoleculeNet', 'PyTorch'],
    install_requires=['requests',
                      'pandas',
                      'numpy',
                      'tqdm',
                      'cloudpickle',
                      'torch',
                      'torch_geometric'],
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Environment :: Console"
    ],
    python_requires='>=3.6',
)


from molnet.molnet_config import molnet_config
# from molnet.load_functions import MolDataset, load_QM7, load_QM7b, load_QM8, load_QM9, load_ESOL, load_FreeSolv, \
#     load_Lipophilicity, load_PCBA, load_MUV, load_HIV, load_BACE, load_BBBP, load_Tox21, load_ToxCast, \
#     load_SIDER, load_ClinTox
import molnet.load_functions
from molnet.splitter import ScaffoldSplitter, RandomSplitter
# from molnet.convert2graph import
from pathlib import Path

__version__ = "0.0.9"

molnet_list = list(molnet_config.keys())

for name in molnet_list:
    fn = getattr(load_functions, f'load_{name}')
    molnet_config[name].add_load_fn(fn)


def load(name, datadir, save_whole_dataset=False, save_split=False, split=None, split_method_override=None, seed=None):
    """

    :param name: Dataset name
    :param datadir: where to store downloaded and extracted and saved data
    :param save_whole_dataset: save pickled dataset, with extracted molecules and target
    :param save_split: save splitting results
    :param split: split ratio, None: default 8/1/1 split; (x, x, x): train/valid/test; (x, x) train/test; int: K-fold cross validation
    :param split_method_override: override default splitter, choice [RandomSplitter, ScaffoldSplitter]
    :param seed: random seed for RandomSplitter, useless when dataset is splitted by scaffold
    :return: molecule datasets
    """
    if split is None:
        split = (0.8, 0.1, 0.1)
        split_method = 'train_valid_test_split'
    elif isinstance(split, (list, tuple)):
        assert len(split) == 3
        split_method = 'train_valid_test_split'
    elif isinstance(split, float):
        split_method = 'train_test_split'
    elif isinstance(split, int):
        split_method = 'k_fold_split'
    else:
        raise TypeError('Argument `split` should be None, list/tuple, int or float')
    assert split_method is not None
    # first check argument, then load data, make error fast
    whole_dataset = molnet_config[name].load_fn(datadir, save=save_whole_dataset)
    if save_split:
        save_split = Path(datadir) / molnet_config[name].relative_path
    else:
        save_split = None
    if molnet_config[name].split == 'random':
        spl = RandomSplitter(save_to=save_split)
    elif molnet_config[name].split == 'scaffold':
        spl = ScaffoldSplitter(save_to=save_split)
    else:
        # TODO: stratidied splitter (normally random split is enough)
        assert molnet_config[name].split == 'stratified'
        spl = RandomSplitter(save_to=save_split)
    # override default splitter
    if split_method_override is not None:
        spl = getattr(molnet.splitter, split_method_override)(save_to=save_split)

    split_fn = getattr(spl, split_method)
    if split_method == 'train_valid_test_split':
        train_dataset, valid_dataset, test_dataset = split_fn(whole_dataset, *split, seed=seed)
        if molnet_config[name].task_type == 'classification':
            train_dataset.calc_weight_classification()
        else:
            train_dataset.cal_weight_basic()
        if valid_dataset is not None:
            valid_dataset.cal_weight_basic()
        if test_dataset is not None:
            test_dataset.cal_weight_basic()
        res = (train_dataset, valid_dataset, test_dataset)

    elif split_method == 'train_test_split':
        train_dataset, test_dataset = split_fn(whole_dataset, frac_train=split, seed=seed)
        if molnet_config[name].task_type == 'classification':
            train_dataset.calc_weight_classification()
        else:
            train_dataset.cal_weight_basic()
        if test_dataset is not None:
            test_dataset.cal_weight_basic()
        res = (train_dataset, test_dataset)
    else:
        assert split_method == 'k_fold_split'
        res = split_fn(whole_dataset, k=split, seed=seed)
        for fold in res:
            if molnet_config[name].task_type == 'classification':
                fold.calc_weight_classification()
            else:
                fold.cal_weight_basic()
    return res




import molnet
from pathlib import Path
import sys
import torch
from torch_geometric.data import Data
from torch_geometric.data import InMemoryDataset
from torch_geometric.data import Batch

import numpy as np
from rdkit import Chem
"""
This script is modified from https://github.com/deepchem/deepchem/blob/master/deepchem/feat/graph_features.py
"""

# def keep_largest_fragment(sml):
#     """
#     Function that returns the SMILES sequence of the largest fragment for a input
#     SMILES sequence.
#     :param str sml: A SMILES sequence.
#     :return: The canonical SMILES sequence of the largest fragment.
#     :rtype: str
#     """
#     mol_frags = Chem.GetMolFrags(Chem.MolFromSmiles(sml), asMols=True)
#     largest_mol = None
#     largest_mol_size = 0
#     for mol in mol_frags:
#         size = mol.GetNumAtoms()
#         if size > largest_mol_size:
#             largest_mol = mol
#             largest_mol_size = size
#     return Chem.MolToSmiles(largest_mol)
#
#
# def remove_salt(sml, remover):
#     """
#     Function that strips salts from a SMILES. :param str sml: A SMILES sequence.
#     :param SaltRemover remover: RDKit's SaltRemover object.
#     :return: The canonical SMILES sequence without salts. If any error on processing, return None instead.
#     :rtype: Union[str, NoneType]
#     """
#     try:
#         sml = Chem.MolToSmiles(remover.StripMol(Chem.MolFromSmiles(sml),
#                                dontRemoveEverything=True))
#         if "." in sml:
#             sml = keep_largest_fragment(sml)
#     except:
#         sml = None
#     return sml


# def one_of_k_encoding(x, allowable_set):
#     if x not in allowable_set:
#         raise Exception("input {0} not in allowable set{1}:".format(x, allowable_set))
#     return [x == s for s in allowable_set]


def one_of_k_encoding_unk(x, allowable_set):
    """Maps inputs not in the allowable set to the last element."""
    if x not in allowable_set:
        x = allowable_set[-1]
    return [x == s for s in allowable_set]


def safe_index(lst, e):
    """Gets the index of e in l, providing an index of len(l) if not found"""
    try:
        return lst.index(e)
    except:
        return len(lst)


def calc_gasteiger_charges(mol_or_atom, iter=12):
    if isinstance(mol_or_atom, Chem.Atom):
        atom = mol_or_atom
    else:
        assert isinstance(mol_or_atom, Chem.Mol)
        atom = mol_or_atom.GetAtomWithIdx(0)
    try:
        mol_or_atom.GetProp('_GasteigerCharge')
    except KeyError:
        mol = atom.GetOwningMol()

        Chem.rdPartialCharges.ComputeGasteigerCharges(mol, nIter=iter, throwOnParamFailure=False)


def atom_features(atom, explicit_H=False, use_chirality=True, gasteiger_charges_iter=12):
    results = one_of_k_encoding_unk(atom.GetSymbol(),
                                    ['B', 'C', 'N', 'O', 'F', 'Si', 'P', 'S', 'Cl', 'Br', 'I', 'other']) + \
              one_of_k_encoding_unk(atom.GetDegree(), [0, 1, 2, 3, 4, 5, 6, 'others']) + \
              [atom.GetFormalCharge(), atom.GetNumRadicalElectrons()] + \
              one_of_k_encoding_unk(atom.GetHybridization(), [
                  Chem.rdchem.HybridizationType.SP, Chem.rdchem.HybridizationType.SP2,
                  Chem.rdchem.HybridizationType.SP3, Chem.rdchem.HybridizationType.SP3D,
                  Chem.rdchem.HybridizationType.SP3D2, 'other']) + [atom.GetIsAromatic()]
    # In case of explicit hydrogen(QM8, QM9), avoid calling `GetTotalNumHs`
    if not explicit_H:
        results = results + one_of_k_encoding_unk(atom.GetTotalNumHs(),
                                                  [0, 1, 2, 3, 4])
    if use_chirality:
        try:
            results += one_of_k_encoding_unk(atom.GetProp('_CIPCode'), ['R', 'S']) + [
                atom.HasProp('_ChiralityPossible')]
        except:
            results += [False, False] + [atom.HasProp('_ChiralityPossible')]
    calc_gasteiger_charges(atom)
    results += [float(atom.GetProp('_GasteigerCharge')), float(atom.GetProp('_GasteigerHCharge'))]
    results = np.array(results)
    results[np.isnan(results)] = 0
    results[np.isinf(results)] = 0
    return results


def bond_features(bond, use_chirality=True):
    bt = bond.GetBondType()
    bond_feats = [
        bt == Chem.rdchem.BondType.SINGLE, bt == Chem.rdchem.BondType.DOUBLE,
        bt == Chem.rdchem.BondType.TRIPLE, bt == Chem.rdchem.BondType.AROMATIC,
        bond.GetIsConjugated(),
        bond.IsInRing()
    ]
    if use_chirality:
        bond_feats = bond_feats + one_of_k_encoding_unk(
            str(bond.GetStereo()),
            ["STEREONONE", "STEREOANY", "STEREOZ", "STEREOE"])
    bond_feats = np.array(bond_feats)
    bond_feats[np.isnan(bond_feats)] = 0
    bond_feats[np.isinf(bond_feats)] = 0
    return bond_feats


def num_atom_features():
    # Return length of feature vector using a very simple molecule.
    m = Chem.MolFromSmiles('CC')
    alist = m.GetAtoms()
    a = alist[0]
    return len(atom_features(a))


def num_bond_features():
    # Return length of feature vector using a very simple molecule.
    simple_mol = Chem.MolFromSmiles('CC')
    Chem.SanitizeMol(simple_mol)
    return len(bond_features(simple_mol.GetBonds()[0]))


def mol_to_pyG_data(mol, explicit_H=False, use_chirality=True, gasteiger_charges_iter=12):
    calc_gasteiger_charges(mol)
    # atoms
    atom_features_list = []
    for atom in mol.GetAtoms():
        atom_feature = atom_features(atom, explicit_H=explicit_H, use_chirality=use_chirality)
        atom_features_list.append(atom_feature)
    x = torch.tensor(np.array(atom_features_list), dtype=torch.float32)
    # bonds, we will force molecules have at least 2 atoms
    edges_list = []
    edge_features_list = []
    for bond in mol.GetBonds():
        i = bond.GetBeginAtomIdx()
        j = bond.GetEndAtomIdx()
        edge_feature = bond_features(bond, use_chirality=use_chirality)
        edges_list.append((i, j))
        edge_features_list.append(edge_feature)
        edges_list.append((j, i))
        edge_features_list.append(edge_feature)

    # data.edge_index: Graph connectivity in COO format with shape [2, num_edges]
    edge_index = torch.tensor(np.array(edges_list).T, dtype=torch.long)

    # data.edge_attr: Edge feature matrix with shape [num_edges, num_edge_features]
    edge_attr = torch.tensor(np.array(edge_features_list),
                             dtype=torch.float32)
    data = Data(x=x, edge_index=edge_index, edge_attr=edge_attr)
    return data


class MoleculeNetGraphDataset(InMemoryDataset):
    def __init__(self, root, transform=None, pre_transform=None, pre_filter=None, dataset='ESOL'):
        assert dataset in molnet.molnet_config.keys() or dataset == 'custom'
        self.dataset = dataset
        self.raw_data = None
        # self.remove_salt = remove_salt
        super(MoleculeNetGraphDataset, self).__init__(root, transform, pre_transform, pre_filter)
        self.data, self.slices = torch.load(self.processed_paths[0])


    @property
    def raw_file_names(self):
        relpath = molnet.molnet_config[self.dataset].relative_path
        files = molnet.molnet_config[self.dataset].files
        return [Path(relpath) / f for f in files]

    @property
    def processed_dir(self):
        return Path(self.root) / 'processed' / molnet.molnet_config[self.dataset].relative_path

    @property
    def processed_file_names(self):
        return [f"{self.dataset}.pt"]

    def download(self):
        mol_dataset = molnet.molnet_config[self.dataset].load_fn(self.raw_dir)
        if molnet.molnet_config[self.dataset].task_type == 'classification':
            mol_dataset.calc_weight_classification()
        else:
            mol_dataset.cal_weight_basic()
        self.raw_data = mol_dataset

    def process(self):
        if self.raw_data is None:
            self.download()
        data_list = [mol_to_pyG_data(mol) for mol in self.raw_data.mols]
        labels, weights = self.raw_data.y, self.raw_data.w
        assert len(data_list) == len(labels) == len(weights)
        for idx, data in enumerate(data_list):
            data.y = torch.tensor(labels[idx])
            data.w = torch.tensor(weights[idx], dtype=torch.float)
        if self.pre_filter is not None:
            data_list = [data for data in data_list if self.pre_filter(data)]

        if self.pre_transform is not None:
            data_list = [self.pre_transform(data) for data in data_list]

        data, slices = self.collate(data_list)
        torch.save((data, slices), self.processed_paths[0])


class CustomMoleculeDataset(InMemoryDataset):
    def __init__(self, root, mols, y, w=None, transform=None, pre_transform=None, pre_filter=None, name='custom'):
        self.raw_data = molnet.load_functions.MolDataset(mols, y)
        self.name = name
        if w is not None:
            self.raw_data.w = w
        else:
            self.raw_data.cal_weight_basic()
        super(CustomMoleculeDataset, self).__init__(root, transform, pre_transform, pre_filter)
        try:
            (Path(self.processed_dir)/'pre_filter.pt').unlink()
            (Path(self.processed_dir) / 'pre_transform.pt').unlink()
        except FileNotFoundError:
            pass
        self.data, self.slices = torch.load(self.processed_paths[0])

    @property
    def raw_file_names(self):
        Path(self.raw_dir).mkdir(parents=True, exist_ok=True)
        file_name_list = list(Path(self.raw_dir).iterdir())
        # assert len(file_name_list) == 1     # currently assume we have a
        # # single raw file
        return file_name_list
    @property
    def processed_file_names(self):
        return [f'{self.name}_{len(self.raw_data)}.pt']

    def download(self):
        print('Dummy download function invoked', file=sys.stderr)

    def process(self):
        data_list = [mol_to_pyG_data(mol) for mol in self.raw_data.mols]
        labels, weights = self.raw_data.y, self.raw_data.w
        assert len(data_list) == len(labels) == len(weights)
        for idx, data in enumerate(data_list):
            data.y = torch.tensor(labels[idx, None])
            data.w = torch.tensor(weights[idx, None], dtype=torch.float)
        if self.pre_filter is not None:
            data_list = [data for data in data_list if self.pre_filter(data)]

        if self.pre_transform is not None:
            data_list = [self.pre_transform(data) for data in data_list]

        data, slices = self.collate(data_list)
        torch.save((data, slices), self.processed_paths[0])

    def normalize(self, std_mean=None):
        if std_mean is not None:
            std, mean = std_mean
        else:
            std = (torch.std(self.data.x, dim=0), torch.std(self.data.edge_attr, dim=0))
            mean = (torch.mean(self.data.x, dim=0), torch.mean(self.data.edge_attr, dim=0))

        self.data.x = (self.data.x - mean[0]) / (std[0]+1e-10)
        self.data.edge_attr = (self.data.edge_attr - mean[1]) / (std[1]+1e-10)
        return std, mean


